=== modified file 'lib/lp/registry/configure.zcml'
--- lib/lp/registry/configure.zcml	2011-01-21 08:12:29 +0000
+++ lib/lp/registry/configure.zcml	2011-02-16 21:34:13 +0000
@@ -87,19 +87,27 @@
 
     <!-- IPersonTransferJob -->
     <securedutility
-        component="lp.registry.model.persontransferjob.MembershipNotificationJob"
-        provides="lp.registry.interfaces.persontransferjob.IMembershipNotificationJobSource">
-        <allow
-            interface="lp.registry.interfaces.persontransferjob.IMembershipNotificationJobSource"/>
-    </securedutility>
-
-    <class class="lp.registry.model.persontransferjob.PersonTransferJob">
-        <allow interface="lp.registry.interfaces.persontransferjob.IPersonTransferJob"/>
-    </class>
-
-    <class class="lp.registry.model.persontransferjob.MembershipNotificationJob">
-        <allow
-            interface="lp.registry.interfaces.persontransferjob.IMembershipNotificationJob"/>
+        component=".model.persontransferjob.MembershipNotificationJob"
+        provides=".interfaces.persontransferjob.IMembershipNotificationJobSource">
+      <allow interface=".interfaces.persontransferjob.IMembershipNotificationJobSource"/>
+    </securedutility>
+
+    <securedutility
+        component=".model.persontransferjob.PersonMergeJob"
+        provides=".interfaces.persontransferjob.IPersonMergeJobSource">
+      <allow interface=".interfaces.persontransferjob.IPersonMergeJobSource"/>
+    </securedutility>
+
+    <class class=".model.persontransferjob.PersonTransferJob">
+        <allow interface=".interfaces.persontransferjob.IPersonTransferJob"/>
+    </class>
+
+    <class class=".model.persontransferjob.MembershipNotificationJob">
+      <allow interface=".interfaces.persontransferjob.IMembershipNotificationJob"/>
+    </class>
+
+    <class class=".model.persontransferjob.PersonMergeJob">
+      <allow interface=".interfaces.persontransferjob.IPersonMergeJob"/>
     </class>
 
     <!-- INameBlacklist -->

=== modified file 'lib/lp/registry/enum.py'
--- lib/lp/registry/enum.py	2011-02-01 04:57:42 +0000
+++ lib/lp/registry/enum.py	2011-02-16 21:34:13 +0000
@@ -80,3 +80,9 @@
 
         Notify affected users of new team membership.
         """)
+
+    MERGE = DBItem(1, """
+        Person merge
+
+        Merge one person or team into another person or team.
+        """)

=== modified file 'lib/lp/registry/interfaces/persontransferjob.py'
--- lib/lp/registry/interfaces/persontransferjob.py	2010-09-30 20:39:02 +0000
+++ lib/lp/registry/interfaces/persontransferjob.py	2011-02-16 21:34:13 +0000
@@ -7,6 +7,8 @@
 __all__ = [
     'IMembershipNotificationJob',
     'IMembershipNotificationJobSource',
+    'IPersonMergeJob',
+    'IPersonMergeJobSource',
     'IPersonTransferJob',
     'IPersonTransferJobSource',
     ]
@@ -78,3 +80,24 @@
     def create(member, team, reviewer, old_status, new_status,
                last_change_comment=None):
         """Create a new IMembershipNotificationJob."""
+
+
+class IPersonMergeJob(IPersonTransferJob):
+    """A Job that merges one person or team into another."""
+
+    from_person = PublicPersonChoice(
+        title=_('Alias for minor_person attribute'),
+        vocabulary='ValidPersonOrTeam',
+        required=True)
+
+    to_person = PublicPersonChoice(
+        title=_('Alias for major_person attribute'),
+        vocabulary='ValidPersonOrTeam',
+        required=True)
+
+
+class IPersonMergeJobSource(IJobSource):
+    """An interface for acquiring IMembershipNotificationJobs."""
+
+    def create(from_person, to_person):
+        """Create a new IMembershipNotificationJob."""

=== modified file 'lib/lp/registry/model/persontransferjob.py'
--- lib/lp/registry/model/persontransferjob.py	2011-01-24 20:10:41 +0000
+++ lib/lp/registry/model/persontransferjob.py	2011-02-16 21:34:13 +0000
@@ -45,14 +45,16 @@
 from lp.registry.interfaces.persontransferjob import (
     IMembershipNotificationJob,
     IMembershipNotificationJobSource,
+    IPersonMergeJob,
+    IPersonMergeJobSource,
     IPersonTransferJob,
     IPersonTransferJobSource,
     )
 from lp.registry.interfaces.teammembership import TeamMembershipStatus
 from lp.registry.model.person import Person
+from lp.services.database.stormbase import StormBase
 from lp.services.job.model.job import Job
 from lp.services.job.runner import BaseRunnableJob
-from lp.services.database.stormbase import StormBase
 
 
 class PersonTransferJob(StormBase):
@@ -321,3 +323,44 @@
                     self.member_template % replacements, force_wrap=True)
                 simple_sendmail(from_addr, address, subject, msg)
         log.debug('MembershipNotificationJob sent email')
+
+
+class PersonMergeJob(PersonTransferJobDerived):
+    """A Job that merges one person or team into another."""
+
+    implements(IPersonMergeJob)
+    classProvides(IPersonMergeJobSource)
+
+    class_job_type = PersonTransferJobType.MERGE
+
+    @classmethod
+    def create(cls, from_person, to_person):
+        """See `IPersonMergeJob`."""
+        return super(PersonMergeJob, cls).create(
+            minor_person=from_person, major_person=to_person, metadata={})
+
+    @property
+    def from_person(self):
+        """See `IPersonMergeJob`."""
+        return self.minor_person
+
+    @property
+    def to_person(self):
+        """See `IPersonMergeJob`."""
+        return self.major_person
+
+    @property
+    def log_name(self):
+        return self.__class__.__name__
+
+    def run(self):
+        """Perform the merge."""
+        from canonical.launchpad.scripts import log
+        log.debug(
+            "%s is about to merge %s into %s", self.log_name,
+            self.from_person.name, self.to_person.name)
+        getUtility(IPersonSet).merge(
+            from_person=self.from_person, to_person=self.to_person)
+        log.debug(
+            "%s has merged %s into %s", self.log_name,
+            self.from_person.name, self.to_person.name)

=== added file 'lib/lp/registry/tests/test_person_merge_job.py'
--- lib/lp/registry/tests/test_person_merge_job.py	1970-01-01 00:00:00 +0000
+++ lib/lp/registry/tests/test_person_merge_job.py	2011-02-16 21:34:13 +0000
@@ -0,0 +1,56 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Tests of `PersonMergeJob`."""
+
+__metaclass__ = type
+
+from zope.component import getUtility
+from zope.interface.verify import verifyObject
+from zope.security.proxy import removeSecurityProxy
+
+from canonical.testing import DatabaseFunctionalLayer
+from lp.registry.interfaces.persontransferjob import (
+    IPersonMergeJob,
+    IPersonMergeJobSource,
+    )
+from lp.testing import TestCaseWithFactory
+
+
+class TestPersonMergeJob(TestCaseWithFactory):
+
+    layer = DatabaseFunctionalLayer
+
+    def setUp(self):
+        super(TestPersonMergeJob, self).setUp()
+        self.from_person = self.factory.makePerson(name='void')
+        self.to_person = self.factory.makeTeam(name='gestalt')
+        self.job_source = getUtility(IPersonMergeJobSource)
+        self.job = self.job_source.create(
+            from_person=self.from_person, to_person=self.to_person)
+
+    def test_interface(self):
+        # PersonMergeJob implements IPersonMergeJob.
+        verifyObject(IPersonMergeJob, self.job)
+
+    def test_properties(self):
+        # PersonMergeJobs have a few interesting properties.
+        self.assertEqual(self.from_person, self.job.from_person)
+        self.assertEqual(self.from_person, self.job.minor_person)
+        self.assertEqual(self.to_person, self.job.to_person)
+        self.assertEqual(self.to_person, self.job.major_person)
+        self.assertEqual({}, self.job.metadata)
+
+    def test_enqueue(self):
+        # Newly created jobs are enqueued.
+        self.assertEqual([self.job], list(self.job_source.iterReady()))
+
+    def test_run(self):
+        # When run it merges from_person into to_person. First we need to
+        # reassign from_person's email address over to to_person because
+        # IPersonSet.merge() does not (yet) promise to do that.
+        from_email = self.from_person.preferredemail
+        removeSecurityProxy(from_email).personID = self.to_person.id
+        removeSecurityProxy(from_email).accountID = self.to_person.accountID
+        self.job.run()
+        self.assertEqual(self.to_person, self.from_person.merged)

